/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;
import java.net.*;
import java.awt.image.*;

/**
 * A class that produces a ImageButton. It is similar to Button, but it
 * has no text label just the image.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class ImageButton extends ImageLabel {

	/** Color of 3D border around image. By default it is set to
	 *  defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;
	
	/** Default color of 3D border around image. Currently a gray
	 *  with R/G/B of 192/192/192. Light grays look best.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
	public static final Color defaultBorderColor = new Color(192, 192, 192);

	/** Default width of 3D border around image. Currently 1.
	 * @see ImageLabel#setBorderWidth
	 * @see ImageLabel#getBorderWidth
	 */
	public static final int defaultBorderWidth = 1;

	/** Determine whether mouse button is now pressed. */
	protected boolean mouseIsDown = false;

	/** Determine whether button is down or up. */
	protected boolean isDown = false;

	/** If propagate is set to true all events are propagate and don't
	 *  catch. This flag is used for xawt component during design time.
	 */
	public boolean propagate = false;

	/** Create an ImageButton with the default image.
	 * @see ImageLabel#defaultImageString
	 */
	public ImageButton() {
		super();
		setDefaults();
	}

	/** Create an ImageButton using the image at URL specified by the string.
	 * @param imageURLString A String specifying the URL of the image.
	 */
	public ImageButton(String imageURLString) {
		super(imageURLString);
		setDefaults();
	}

	/** Create an ImageButton using the image at URL specified.
	 * @param imageURL The URL of the image.
	 */
	public ImageButton(URL imageURL) {
		super(imageURL);
		setDefaults();
	}

	/** Create an ImageButton using the image specified. You would only want to
	 *  call it directly if you already have an image (e.g. created via
	 *  createImage).
	 * @param image The image.
	 */
	public ImageButton(Image image) {
		super(image);
		setDefaults();
	}

	private void setDefaults() {
		setBorderWidth(defaultBorderWidth);
		setBorderColor(defaultBorderColor);
		propagate = false;
	}

	/** Sets the border color.
	 * @param borderColor New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
	public void setBorderColor(Color borderColor) {
		marginColor = borderColor;
	}

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
	public Color getBorderColor() {
		return(marginColor);
	}

	/** Call super setMargins and increase left and top or right and bottom
	 *  margin whether button is down or not.
	 * @see ImageLabel#margin
	 * @see ImageLabel#marginL
	 * @see ImageLabel#marginR
	 * @see ImageLabel#marginT
	 * @see ImageLabel#marginB
	 * @see ImageLabel#setMargins
	 */
	protected void setMargins() {
		super.setMargins();
		if (isDown) {
			marginL++;
			marginT++;
		}
		else {
			marginR++;
			marginB++;
		}
	}

	public void paint(Graphics g) {
		setMargins();
		super.paint(g);
		drawBorder(g);
	}

	public boolean mouseDown(Event event, int x, int y) {
		if (propagate)
			return false;
		mouseIsDown = true;
		isDown = true;
		repaint();
		return false;
	}

	public boolean mouseUp(Event event, int x, int y) {
		if (propagate)
			return false;
		mouseIsDown = false;
		if (isDown) {
			isDown = false;
			event.id = Event.ACTION_EVENT;
			event.arg = (Object)image;
		}
		repaint();
		return false;
	}

	public boolean mouseExit(Event event, int x, int y) {
		if (propagate)
			return false;
		if (mouseIsDown) {
			isDown = false;
			repaint();
		}
		return false;
	}

	public boolean mouseEnter(Event event, int x, int y) {
		if (propagate)
			return false;
		if (mouseIsDown) {
			isDown = true;
			repaint();
		}
		return false;
	}

	/** Draws 3D border around the image. */
	protected void drawBorder(Graphics g) {
		Dimension d = size();
		int left = 0;
		int top = 0;
		int width = d.width - 1;
		int height = d.height - 1;

		g.setColor(Color.black);
		if (!isDown) {
			g.drawLine(0, height, width, height);
			g.drawLine(width, 0, width, height);
		}
		else {
			g.drawLine(0, 0, width, 0);
			g.drawLine(0, 0, 0, height);
			left++;
			top++;
		}
		width--;
		height--;

		g.setColor(marginColor);
		for(int i = 0; i < margin; i++) {
			g.draw3DRect(left, top, width, height, !isDown);
			left++;
			top++;
			width = width - 2;
			height = height - 2;
		}
	}
}