/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces several basic geometric objects.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class Shape extends Canvas {

	/** Circle constant is used for specification the shape.
	 * @see #shape
	 */
	public final static int SHAPE_CIRCLE = 1;

	/** Elipse constant is used for specification the shape.
	 * @see #shape
	 */
	public final static int SHAPE_ELIPSE = 2;
	
	/** Rectangle constant is used for specification the shape.
	 * @see #shape
	 */
	public final static int SHAPE_RECTANGLE = 3;

	/** Square constant is used for specification the shape.
	 * @see #shape
	 */
	public final static int SHAPE_SQUARE = 4;
    
	/** Current shape. Allowed values are constants SHAPE_xxxx.
	 *  For setting the value use setStyle method.
	 * @see #SHAPE_CIRCLE
	 * @see #SHAPE_ELIPSE
	 * @see #SHAPE_RECTANGLE
	 * @see #SHAPE_SQUARE
	 * @see #defaultShape
	 * @see #setShape
	 * @see #getShape
	 */
	protected int shape;

	/** Default value of shape variable. When the Shape is created
	 *  shape is set to this value. 
	 * @see #SHAPE_CIRCLE
	 * @see #shape
	 */
	public static final int defaultShape = SHAPE_CIRCLE;

	/** Determine whether the shape is filled or blank.
	 * see #defaultFill
	 * see #getFill
	 * see #setFill
	 */
    protected boolean fill;

	/** Default value of fill variable.
	 * see #fill
	 * see #getFill
	 * see #setFill
	 */
	public static final boolean defaultFill = true;

	/** The basic color of the border. By default it is set 
	 * to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently black.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
    public static final Color defaultBorderColor = Color.black;

	/** Current width of the border around the component.
	 * @see #defaultBorderWidth
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */	
	protected int margin = 0;

	/** Default value of margin. Currently 1.
	 * @see #margin
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */
    public static final int defaultBorderWidth = 1;

	/** This color is used for filling the shape.
	 * @see #defaultFillColor
	 * @see #fill
	 * @see #getFill
	 * @see #setFill
	 * @see #getFillColor
	 * @see #setFillColor
	 */
    protected Color fillColor = null;
	/** Default value of fillColor.
	 * @see #fillColor
	 * @see #getFillColor
	 * @see #setFillColor
	 */
    public static final Color defaultFillColor = Color.white;

	/** Create new Shape component with all default values. */
	public Shape() {
		setDefaults();
	}

	private void setDefaults() {
		shape = defaultShape;
    	marginColor = defaultBorderColor;
		margin = defaultBorderWidth;
    	fillColor = defaultFillColor;
		fill = defaultFill;
	}

	/** Sets the new shape. Allowed values are all Shape.SHAPE_XXX constants.
	 * @param aShape new value of shape.
	 * @see #shape
	 * @see #getShape
	 * @see #SHAPE_CIRCLE
	 * @see #SHAPE_ELIPSE
	 * @see #SHAPE_RECTANGLE
	 * @see #SHAPE_SQUARE
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 */
	public void setShape(int aShape) throws ArrayIndexOutOfBoundsException {
		if (aShape == shape)
			return;
		if ((aShape < 1) || (aShape > 4))
	        throw new ArrayIndexOutOfBoundsException();
		shape = aShape;
		repaint();
	}

	/** Gets the current value of shape.
	 * @return Current shape.
	 * @see #shape
	 * @see #setShape
	 * @see #SHAPE_CIRCLE
	 * @see #SHAPE_ELIPSE
	 * @see #SHAPE_RECTANGLE
	 * @see #SHAPE_SQUARE
	 */
	public int getShape() {
		return shape;
	}

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
        repaint();
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return marginColor;
    }

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see #margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
        repaint();
    }

	/** Gets border width around the component.
	 * @see #margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
        return(margin);
    }

	/** Sets new value of fill flag.
	 * @param aFill New value of fill.
	 * @see #defaultFillColor
	 * @see #fill
	 * @see #getFill
	 */
    public void setFill(boolean aFill) {
        fill = aFill;
        repaint();
    }

	/** Gets current value of fill flag.
	 * @return Current value of fill.
	 * @see #defaultFillColor
	 * @see #fill
	 * @see #setFill
	 */
    public boolean getFill() {
        return(fill);
    }

	/** Sets the fill color.
	 * @param col New value of fillColor.
	 * @see #fillColor
	 * @see #defaultFillColor
	 * @see #getFillColor
	 */
    public void setFillColor(Color col) {
		fillColor = col;
		repaint();
    }

	/** Gets the fill color.
	 * @return Current fill color.
	 * @see #fillColor
	 * @see #defaultFillColor
	 * @see #setFillColor
	 */
    public Color getFillColor() {
        return fillColor;
    }

	public void paint(Graphics g) {
		Dimension d = size();

		int x = 0;
		int y = 0;
		int w = d.width - 1;
		int h = d.height - 1;
		
		g.setColor(marginColor);
		switch (shape) {
			case SHAPE_CIRCLE: 	if (w > h) {x += (w-h)/2; w = h;}
								else { y += (h-w)/2; h = w; }
			case SHAPE_ELIPSE:	g.fillOval(x,y,w,h);
								break;
			case SHAPE_SQUARE: 	if (w > h) {x += (w-h)/2; w = h;}
								else { y += (h-w)/2; h = w; }
			case SHAPE_RECTANGLE:	g.fillRect(x,y,w,h);
									break;
		}
		w -= 2*margin;
		h -= 2*margin;
		x += margin;
		y += margin;

		if (fill)
			g.setColor(fillColor);
		else
			g.setColor(getBackground());
		switch (shape) {
			case SHAPE_CIRCLE:
			case SHAPE_ELIPSE:	g.fillOval(x,y,w,h);
								break;
			case SHAPE_RECTANGLE:
			case SHAPE_SQUARE: 	g.fillRect(x,y,w,h);
								break;
		}
	}

    public Dimension minimumSize() {
        return new Dimension(50,50);
    }

    public Dimension preferredSize() {
        return new Dimension(50,50);
    }
}
