package xelfi.awt;

import java.awt.*;

import xelfi.util.Debug;

/**
 * Owner draw list box in panel, with scrollbars.
 * Automatically handles scrolling and extended selections.
 * (Shift+arrows, Shift or Ctrl+ mouse)
 * Delivers most of its functionality to the XFListViewCanvas class.
 *
 * @version 1.00, 2 Feb 1997
 * @author Dafe
	Last change:  DF    2 Feb 97    4:09 pm
*/
public class XFListViewPanel extends Panel implements XelfiScrollableOwner {

	private XFListViewCanvas	listViewCanvas;

	// scrollbar movement variables
	private Scrollbar					vScroll;
	private int								lineIncrement = 10;

	/**
	 * Creates new empty signle sel. list
	*/
  public XFListViewPanel () {
		this(false);
	}

	/**
	 * Creates new list with multiple selections 
	 * enabled/disabled as specified in param.
	*/
  public XFListViewPanel (boolean multipleSelections) {
		listViewCanvas = new XFListViewCanvas(multipleSelections);
		listViewCanvas.setScrollableOwner(this);
		vScroll = new Scrollbar(Scrollbar.VERTICAL);
		vScroll.hide();
		setLayout(new BorderLayout());
		add("Center", listViewCanvas);
		add("East", vScroll);
	}

  /**
	 * Adds the specified item to the end of scrolling list.
	 * Only reference to the item is added to the list.
	*/
	public void addItem (XelfiListViewElement item) {
		listViewCanvas.addItem(item);
		refreshScrollbar();
	}

	/**
	 * Adds the specified item to the specified index.
	 * Indexes are counted from 0 base.
	 * Only reference to the item is added to the list.
	*/
	public void addItem (XelfiListViewElement item, int index) {
		listViewCanvas.addItem(item, index);
		refreshScrollbar();
	}

	/**
	 * Returns true if this list allows multiple selections.
	*/
	public boolean allowsMultipleSelections () {
		return listViewCanvas.allowsMultipleSelections();
	}

	/**
	 * Clears the list.
	*/
	public void clear () {
		listViewCanvas.clear();
		refreshScrollbar();
	}

	/**
	 * Returns the number of items in the list.
	*/
	public int countItems () {
		return listViewCanvas.countItems();
	}

	/**
	 * Returns the number of selected items in the list.
	*/
	public int countSelectedItems () {
		return listViewCanvas.countSelectedItems();
	}

	/**
	 * Delete an item from the list.<br>
	 * Item is specified by its index, 
	 * counted from 0 for first item in the list.
	 * Returns false if specified item couldn't be deleted;
	 * true otherwise.
	*/
  public boolean delItem (int index) {
		boolean result = listViewCanvas.delItem(index);
		if (result) {
			refreshScrollbar();
		}
		return result;
	}

	/**
	 * Deselects the item at the specified index.
	 * Returns false if specified item hasn't been selected;
	 * true otherwise.
	*/
	public boolean deselect (int index) {
		return listViewCanvas.deselect(index);
	}

	/**
	 * Gets the item associated with the specified index.
	*/
	public XelfiListViewElement getItem (int index) {
		return listViewCanvas.getItem(index);
	}

	/**
	 * Get the selected item on the list or -1 if no item is selected.
	 * If multiple selections is set, returns index of 
	 * first selected item.
	*/
	public int getSelectedIndex () {
		return listViewCanvas.getSelectedIndex();
	}

	/**
	 * Returns the selected indexes on the list.
	*/
	public int[] getSelectedIndexes () {
		return listViewCanvas.getSelectedIndexes();
	}

	/**
	 * Returns the selected item on the list 
	 * or null if no item is selected.
	*/
	public XelfiListViewElement getSelectedItem () {
		return listViewCanvas.getSelectedItem();
	}

	/**
	 * Returns the selected items on the list.
	*/
	public XelfiListViewElement[] getSelectedItems () {
		return listViewCanvas.getSelectedItems();
	}

	/**
	 * Returns true if the item at the specified index has been selected; 
	 * false otherwise.
	*/
	public boolean isSelected (int index) {
		return listViewCanvas.isSelected(index);
	}

	/**
	 * Replaces the item at the given index.
	*/
	public void replaceItem (XelfiListViewElement item, int index) {
		listViewCanvas.replaceItem(item, index);
	}

	/**
	 * Selects the item at the specified index.
	*/
	public boolean select (int index) {
		return listViewCanvas.select(index);
	}

	/**
	 * Conditionally selects or unselects item at the specified index.
	*/
	public void select (int index, boolean selectItem) {
		listViewCanvas.select(index, selectItem);
	}

	/**
	 * Enables/disables multiple selections for list.
	*/
	public boolean setMultipleSelections (boolean multiple) {
		return listViewCanvas.setMultipleSelections(multiple);
	}

	/**
	 * Returns index of an item which contains specified point.
	 * Point coordinates are relative to the getView() point.
	 * Returns -1 if no such item can be found.
	*/
	public int getIndexFromPoint (int x, int y) {
		return listViewCanvas.getIndexFromPoint(x, y);
	}

	/**
	 * Returns absolute coordinates of the upper left point
	 * of the item at index.
	*/
	public Point getUpperLeft (int index) {
		return listViewCanvas.getUpperLeft(index);
	}

	/**
	 * Makes na item with the specified index visible
	 * in the canvas.
	*/
	public void makeVisible (int index) {
		listViewCanvas.makeVisible(index);
	}

	/**
	 * Access method. Returns horizontal indent.
	*/
	public int getYIndent () {
		return listViewCanvas.getYIndent();
	}

	/**
	 * Access method. Returns vertical indent.
	*/
	public int getXIndent () {
		return listViewCanvas.getXIndent();
	}

	/**
	 * Access method. Returns horizontal space between items.
	*/
	public int getSpaceBetween () {
		return listViewCanvas.getSpaceBetween();
	}

	/**
	 * Access method. Sets horizontal indent.
	*/
	public void setYIndent (int yIndent) {
		listViewCanvas.setYIndent(yIndent);
		refreshScrollbar();
	}

	/**
	 * Access method. Sets vertical indent.
	*/
	public void setXIndent (int xIndent) {
		listViewCanvas.setXIndent(xIndent);
		refreshScrollbar();
	}

	/**
	 * Access method. Sets horizontal space between items.
	*/
	public void setSpaceBetween (int spaceBetween) {
		listViewCanvas.setSpaceBetween(spaceBetween);
		refreshScrollbar();
	}

	/**
	 * Paints list. Calls paintArea with component's size.
	*/
	public void paint (Graphics g) {
		listViewCanvas.paint(g);
	}

	/**
	 * Handles incoming events.
	*/
	public boolean handleEvent (Event e) {
		boolean result = super.handleEvent(e);
    if (e.target == vScroll)
    {
			Point view = listViewCanvas.getView();
			listViewCanvas.setView(view.x,
														 vScroll.getValue()*lineIncrement);
    }
		// propagate
		return result;
	}

	// Now implement methods of XelfiScrollableOwner
	/**
	 * Notifies that visible area was changed.
	*/
	public void visibleAreaChanged () {
		refreshScrollbarPosition();
	}

	/**
	 * Notifies that whole size of scrollable object
	 * was changed (due to inserting, deleting, for example).
	*/
	public void wholeSizeChanged () {
		refreshScrollbar();
	}

	public XFListViewCanvas getListViewCanvas () {
		return listViewCanvas;
	}

	private void refreshScrollbarPosition () {
		Point view = listViewCanvas.getView();
		vScroll.setValue(view.y/lineIncrement);
	}

	private void refreshScrollbar () {
		Point view = listViewCanvas.getView();
		int visibleHeight = listViewCanvas.visibleSize().height;
		int listHeight = listViewCanvas.wholeSize().height;
		Debug.debug(Debug.INSPECTOR, "Visible height: "+visibleHeight);
		Debug.debug(Debug.INSPECTOR, "Whole height: "+listHeight);
		boolean showScrollbar = listHeight > visibleHeight;
		if (showScrollbar) {
			vScroll.setValues(view.y/lineIncrement,
												visibleHeight/lineIncrement+1,
												0, (listHeight-visibleHeight)/lineIncrement+1);
			vScroll.setLineIncrement(lineIncrement);
		}
		if ((vScroll.isShowing() && !showScrollbar) ||
				(!vScroll.isShowing() && showScrollbar)) {
			vScroll.show(showScrollbar);
			layout();
			if (!showScrollbar) {
			  listViewCanvas.setView(view.x, 0);
			}
		}
	}

}

