
package xelfi.browser.util;
import java.awt.*;
/**
 * Flow layout is used to layout buttons in a panel. It will arrange
 * buttons left to right until no more buttons fit on the same line.
 * Each line is centered.
 *
 * @version 	1.18, 14 Dec 1995
 * @author 	Arthur van Hoff
 * @author 	Sami Shaio
 */
public class HorizLayout implements LayoutManager {

    /**
     * The left alignment variable.
     */
    public static final int LEFT 	= 0;

    /**
     * The left alignment variable.
     */
    public static final int TOP 	= 4;

    /**
     * The right alignment variable.
     */
    public static final int CENTER 	= 1;

    /**
     * The right alignment variable.
     */
    public static final int RIGHT 	= 2;

    /**
     * The right alignment variable.
     */
    public static final int BOTTOM 	= 5;

    /**
     * The right alignment variable.
     */
    public static final int RESIZE 	= 3;

    int alignH;
    int alignV;
    int hgap;
    int vgap;

    /**
     * Constructs a new Flow Layout with a centered alignment.
     */
    public HorizLayout() {
	this(CENTER,CENTER, 0, 0);
    }

    /**
     * Constructs a new Flow Layout with the specified alignment.
     * @param align the alignment value
     */
    public HorizLayout(int alignH,int alignV) {
	this(alignH,alignV, 5, 5);
    }

    /**
     * Constructs a new Flow Layout with the specified alignment and gap
     * values.
     * @param align the alignment value
     * @param hgap the horizontal gap variable
     * @param vgap the vertical gap variable
     */
    public HorizLayout(int alignH,int alignV, int hgap, int vgap) {
	this.alignH = alignH;
	this.alignV = alignV;
	this.hgap = hgap;
	this.vgap = vgap;
    }

    /**
     * Adds the specified component to the layout. Not used by this class.
     * @param name the name of the component
     * @param comp the the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Removes the specified component from the layout. Not used by
     * this class.
     * @param comp the component to remove
     */
    public void removeLayoutComponent(Component comp) {
    }

    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target)
    {
    	Dimension dim = new Dimension(0, 0);
    	int nmembers = target.countComponents();

    	for (int i = 0 ; i < nmembers ; i++)
    	{
    	    Component m = target.getComponent(i);
    	    if( m.isVisible() )
    	    {
        		Dimension d = m.preferredSize();
        		dim.width = Math.max(dim.width, d.width);
        		if( i > 0 )
        		{
        		    dim.height += hgap;
        		}
        		dim.height += d.height;
        		//System.out.print("y = " + d.height);
    	    }//else
                //System.out.print("Not visible!");

    	}
    	Insets insets = target.insets();
    	dim.width += insets.left + insets.right + hgap*2;
    	dim.height += insets.top + insets.bottom + vgap*2;

        //System.out.print("lay sh pref :" + dim.width + "," + dim.height);
        //System.out.print("lay sh ins :l " + insets.left + ",r " + insets.right);
        //System.out.println("lay sh ins :t " + insets.top + ",b " + insets.bottom);

    	return dim;
    }

    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container.
     * @param target the component which needs to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target)
    {
    	Dimension dim = new Dimension(0, 0);
    	int nmembers = target.countComponents();

    	for (int i = 0 ; i < nmembers ; i++)
    	{
    	    Component m = target.getComponent(i);
    	    if (m.isVisible())
    	    {
        		Dimension d = m.minimumSize();
        		dim.width = Math.max(dim.width, d.width);
        		if (i > 0)
        		{
        		    dim.height += hgap;
        		}
        		dim.height += d.height;
    	    }
    	}
    	Insets insets = target.insets();
    	dim.width += insets.left + insets.right + hgap*2;
    	dim.height += insets.top + insets.bottom + vgap*2;
    	return dim;
    }

    /**
     * Centers the elements in the specified row, if there is any slack.
     * @param target the component which needs to be moved
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width dimensions
     * @param height the height dimensions
     * @param rowStart the beginning of the row
     * @param rowEnd the the ending of the row
     */
     // height kolik zbyva volneho mista na y

    private void moveComponents(Container target, int x, int y, int width, int height, int rowStart, int rowEnd)
    {
	    switch (alignV)
	    {
	        case TOP:
	            break;
	        case CENTER:
	            y += height / 2;
	            break;
    	    case BOTTOM:
	            y += height;
	            break;
	    }
	    for (int i = rowStart ; i < rowEnd ; i++)
	    {
	        Component m = target.getComponent(i);
	        if (m.isVisible())
	        {
        	    switch (alignH)
	            {
        	        case LEFT:
        	            m.move(x, y);
	                    break;
        	        case CENTER:
	                    m.move(x + (width - m.size().width)/2, y);
	                    break;
    	            case RIGHT:
	                    m.move(x + width - m.size().width, y);
	                    break;
	                case RESIZE:
	                    m.reshape( x, y, width, m.preferredSize().height);
	                    break;
	            }

		        y += vgap + m.size().height;
	        }
	    }
    }

    /**
     * Lays out the container. This method will actually reshape the
     * components in the target in order to satisfy the constraints of
     * the BorderLayout object.
     * @param target the specified component being laid out.
     * @see Container
     */
    public void layoutContainer(Container target) {
	Insets insets = target.insets();
	int maxheight = target.size().height - (insets.top + insets.bottom + vgap*2);
	int nmembers = target.countComponents();
	int x = insets.left + hgap, y = 0;
	int roww = 0, start = 0;
	//System.out.print("maxheight = " + maxheight);

	for (int i = 0 ; i < nmembers ; i++)
	{
	    Component m = target.getComponent(i);
    	//System.out.print("y = " + y);
	    if (m.isVisible())
	    {
		    Dimension d = m.preferredSize();
		    if( alignH != RESIZE )
		        m.resize(d.width, d.height);

    		if ((y == 0) || ((y + d.height) <= maxheight))
    		{
		        if (y > 0)
		        {
        			y += vgap;
		        }
		        y += d.height;
		        roww = Math.max(roww, d.width);
		    }else
		    {
            	//System.out.print("move ");
		        moveComponents(target, x, insets.top + vgap, roww, maxheight - y, start, i);
    		    y = d.height;
	    	    x += hgap + roww;
		        roww = d.width;
		        start = i;
		    }
	    }
	}
   	//System.out.print("move ");
	moveComponents(target, x, insets.top + vgap, roww, maxheight - y, start, nmembers);
   	//System.out.println(" e ");
    }

    /**
     * Returns the String representation of this FlowLayout's values.
     */
    public String toString() {
	String str = "";
	switch (alignH) {
	  case LEFT:    str = ",align=left"; break;
	  case CENTER:  str = ",align=center"; break;
	  case RIGHT:   str = ",align=right"; break;
	}
	return getClass().getName() + "[hgap=" + hgap + ",vgap=" + vgap + str + "]";
    }
}
