/**
 * Breakpoint
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import xelfi.editor.Source;
import xelfi.editor.Document;
import xelfi.editor.Position;
import xelfi.editor.PositionInfo;
import xelfi.top.TopLevel;

/**
 * This is a class which is stored in the document class Breakpoints.
 * It contains information about setting of a breakpoint (position
 * in source file and the breakpoint state - enabled, disabled, invalid).
 */

class Breakpoint
{
	private String className;
	private int lineNumber;
	private boolean enabled;
	private boolean valid;

	/**
	 * Breakpoint constructor.
	 */
	Breakpoint(String className, int lineNumber)
	{
		this.className = className;
		this.lineNumber = lineNumber;
		this.enabled = true;
		this.valid = true;
	}

	/**
	 * Returns a name of the class in which the breakpoint is set.
	 */
	String getClassName()
	{
		return className;
	}

	/**
	 * Returns a number of the line on which the breakpoint is set.
	 */
	int getLineNumber()
	{
		return lineNumber;
	}

	/**
	 * Converts the line from "the editor line number" to "snapshot line number"
	 * If it is not possible to convert it will left "the editor line number"
	 * in field lineNumber of this class.
	 *
	 * Note: 'The snapshot line numbers" are used by Debugger (or more exactly
	 * by RemoteDebugger).
	 */
	void convertLineNumber()
	{
		try
		{
			Source s = new Source(className, false);
			if (s == null)
				return;

			Document d = s.getDocument();
			if (d == null)
				return;

			Position p = d.goToLine(lineNumber-1);
			if (p == null)
				return;

			PositionInfo pi = TopLevel.getCurrentSnapshot().getPositionInfo(p);
			if (pi == null)
				return;

			lineNumber = pi.getLine()+1;
		}
		catch (Exception e)
		{
			return;
		}
	}

	/**
	 * Returns a number of the line (in the editor window) on which
	 * the breakpoint is set. (Line numbers in Editor and that ones used
	 * by Debugger may differ, if user edits the source code while debugging).
	 * The current snapshot (from the Top) is used for this purpose.
	 *
	 * If it is not possible to get appropriate information from the editor,
	 * only the original line number is returned.
	 *
	 * Note: Lines are counted from 1.
	 */
	int getEditorLineNumber()
	{
		try
		{
			Source s = new Source(className, false);
			if (s == null)
				return -1;

			Position p = TopLevel.getCurrentSnapshot().getPosition(s, lineNumber-1).beginOfLine();
			if (p == null)
				return -1;

			Document d = s.getDocument();
			if (d == null)
				return -1;

			return d.lineToInt(p)+1;
		}
		catch (Exception e)
		{
			return lineNumber;
		}
	}
  
	/**
	 * Sets the line number.
	 */
	void setLineNumber(int lineNumber)
	{
		this.lineNumber = lineNumber;
	}

	/**
	 * Sets the line number and the class name.
	 */
	void setClassNameAndLineNumber(String className, int lineNumber)
	{
		this.className = className;
		this.lineNumber = lineNumber;
	}

	/**
	 * Changes the breakpoint stete to enabled (only if the breakpoint is valid).
	 */
	void setEnabled()
	{
		if (valid)
			enabled = true;
	}

	/**
	 * Changes the breakpoint state to disabled (only if the breakpoint is valid).
	 */
	void setDisabled()
	{
		if (valid)
			enabled = false;
	}

	/**
	 * Returns the enabled flag.
	 */
	boolean isEnabled()
	{
		return enabled;
	}

	/**
	 * Changes the breakpoint state to valid.
	 */
	void setValid()
	{
		valid = true;
	}

	/**
	 * Cbanges the breakpoint state to invalid.
	 */
	void setInvalid()
	{
		valid = false;
	}

	/**
	 * Returns the valid flag.
	 */
	boolean isValid()
	{
		return valid;
	}

	/**
	 * Returns the string description of the breakpoint in tbe form:
	 *   className:lineNumber : <enabled | disabled | invalid>
	 */
	public String toString()
	{
		String string;

		string = className+":"+getEditorLineNumber()+" : ";
		if (valid)
			string += enabled ? "enabled" : "disabled";
		else
			string += "invalid";

		return string;
	}
}
