/**
* Xelfi
*/

package xelfi.design.components;

import java.awt.Frame;
import java.awt.Container;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * XelfiDesignLayout Interface is a common interface for layout managers that can be used 
 * during Xelfi design time.<p>
 * Layout managers are, in some way, also components. They have properties which are shown
 * in ComponentInspector. However it behaves different way - there is one XelfiDesignLayout
 * per XelfiVisualContainer (even for null Layout there is a design class). This layout 
 * keeps list of all components that are to be layout managed together with all its layout 
 * settings (i.e. layout's properties);
 * @version 	1.51, 27 Feb 1997
 *				from 1.20 has viewer types methods
 *				from 1.30 has layoutAdd methods
 *				from 1.31 has a allowsDragging method
 *				from 1.32 has a getClassName method
 *				from 1.40 all methods that had "String compName" as param now have "XelfiVisualComponent comp"
 *							(it must be visual component!), setProperty returns value
 *				from 1.41 neededImports()
 *				from 1.42 component list management with XVC instead of String
 *				from 1.50 has setParent method
 * @author 	Ian Formanek
 */
public interface XelfiDesignLayout{

	/**
	 * Method which allows layout to acquire parent container 
	 * (the container that this layout manages).
	 * This is necessary for some layouts - e.g. in loadFromStream, ...
	 * @param aParent		parent container
	 */
	public void setParent(XelfiVisualContainer aParent);

	/**
	 * Method to get number of properties of this layout.
	 * @return		number of properties of this layout
	 * @see #getPropertyNames
	 * @see #getProperty
	 * @see #setProperty
	 */
	public int getPropertyCount();

	/**
	 * Method to get names of properties of this layout.
	 * @return		array of names of properties
	 * @see #getPropertyCount
	 * @see #getProperty
	 * @see #setProperty
	 */
	public String[] getPropertyNames();

	/**
	 * Method to get constant that describes what viewer to use in ComponentInspector for
	 *	given property (the constants are defined in XFDesignConstants)
	 * @param	index		the index of property
	 * @return				the constant specifying what viewer to use
	 * @exception XFPropertyIndexOutOfBoundsException
	 * 		should be thrown when Index value is out of range 
	 *			<tt>((index < 0) || (index > property count-1))</tt>
	 * @see #getPropertyValuesArray
	 */
	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException;

	/**
	 * Method to get array of textual representation of all possible values for given property.
	 * This is only for properties with viewer type PVT_CHOICE.
	 * @param 	index	the index of property
	 * @return		the array of string value names
	 * @exception XFPropertyIndexOutOfBoundsException
	 * 		should be thrown when Index value is out of range 
	 *			<tt>((index < 0) || (index > property count-1))</tt>
	 *			or when the given property does not have a viewer type PVT_CHOICE
	 * @see #getPropertyViewerType
	 */
	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException;

	/**
	 * Method to get array of textual representation of all possible values for given property.
	 * This is only for properties with viewer type PVT_CHOICE.
	 * @param 	index	the index of property
	 * @return		the array of possible valuesvalues 
	 * @exception XFPropertyIndexOutOfBoundsException
	 * 		should be thrown when Index value is out of range 
	 *			<tt>((index < 0) || (index > property count-1))</tt>
	 *			or when the given property does not have a viewer type PVT_CHOICE
	 * @see #getPropertyViewerType
	 */
	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException;

	/**
	 * Method to get property value for given component and property index.
	 * @param 		comp			component which property value to return
	 * @param 		index			index of property
	 * @return		property value
	 * @see #setProperty
	 */
	public Object getProperty(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException;
		
	/**
	 * Method to get textual representation of property value for given component 
	 * and property index.
	 * @param 		comp			component which property value to return
	 * @param 		index			index of property
	 * @return		textual representation of property value
	 */
	public String getPropertyString(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException;
		
	/**
	 * Method to set property value for given component and property index.
	 * @param 		comp			component which property value to set
	 * @param 		index			index of property
	 * @param		value			property value
	 * @return						return types defined in XFDesignConstants (same as setProperty in XelfiVisualComponent)
	 * @see #getProperty
	 */
	public int setProperty(XelfiVisualComponent comp, int index, Object value) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException,
		XFNameNotFoundException;

	/**
	 * Method to invoke specialInput dialog on given property value for given component.
	 * @param 		parent		parent of the dialog
	 * @param		comp			component which property to change
	 * @param 		index			index of property
	 */
	public void specialPropertyInput(Frame parent, XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyNoSpecialInputException,
		XFNameNotFoundException;
		
	/**
	* Method to get name of the LAYOUT class (e.g. for XFlowLayout extends FlowLayout it will 
	* return "FlowLayout")
	* @return	name of the layout class
	*/
	public String getClassName();

	
	///////////////////
	// Layout Things //
	///////////////////

	/**
	 * This method sas whether this layout supports component dragging, that is whether the position
	 * of components is significant. Currently, only AbsoluteLayout returns true, none of the awt 
	 * layout returns true.
	 * @return 	whether this layout supports component dragging (i.e. component position has a meaning)
	 */
	public boolean allowsDragging();

	/**
	 * Adds given component to given container with preset layout properties.
	 * If the component is not found in list, it is added with default settings and entry for it is
	 * created in the XLayout internal tables.
	 * @param 	cont	Container to which to add
	 * @param 	comp	component to add to this layout
	 */
	public void layoutAdd(Container cont, XelfiVisualComponent comp);

	/**
	 * Adds given component to given container with preset layout properties.
	 * If the component is not found in list, it is added with default settings and entry for it is
	 * created in the XLayout internal tables.
	 * @param 	cont	Container to which to add
	 * @param 	pos	position to which to add
	 * @param 	comp	component to add to this layout
	 */
	public void layoutAdd(Container cont, int pos, XelfiVisualComponent comp);

	///////////////////////////////
	// Component list management //
	///////////////////////////////

	public void addComponent(XelfiVisualComponent comp);
	public void renameComponent(XelfiVisualComponent comp, String newName) throws 
		XFNameNotFoundException;
	public void removeComponent(XelfiVisualComponent comp) throws 
		XFNameNotFoundException;

	/////////////////////
	// Code generation //
	/////////////////////
	
	/**
	* Method to find out what a component needs to import (or what should be generated into
	* imports section of generated source of form that uses this component).
	* It returns array of strings. Each of these strings corresponds with one import line.
	* The string is only what should be inside import (e.g. only "java.awt.*" for "import java.awt.*;")
	* @return		array of strings that appear in the imports
	*/
	public String[] neededImports();

	/**
	* Returns whether it is necessary to generate instance variable for this layout.
	* Not used in current version of Xelfi.
	* @return		true it is necessary to generate instance variable for this layout, false otherwise
	*/
	public boolean genLayoutVariable();

	/**
	* Generates initialisation code for this layout - i.e. cteate new instance of layout and
	* set common layout properties.
	* @param	containerName	name of container that is managed by this layout
	* @param	varName		name of instance variable of this layout
	* @return		generated code
	*/
	public String genLayoutInitCode(String containerName, String varName);
		
	/**
	* Generates add code for given component. This is typically just "add(...);"
	* @param	containerName	name of container that is managed by this layout
	* @param	comp			component which add code should be generated
	* @param	varName		name of instance variable of this layout
	* @return		generated code
	*/
	public String genComponentLayoutCode(String containerName, XelfiVisualComponent comp, String varName);
	
	/////////////////
	// persistence //
	/////////////////
	
	/**
	 * Method which loads this layout from stream (i.e. properties of all components it manages).
	 * @param stream		stream to load layout from
	 */
	public void loadFromStream(DataInput stream) throws
		IOException;
	
	/**
	 * Method which saves this layout to stream (i.e. properties of all components it manages).
	 * @param stream		stream to save layout to
	 */
	public void saveToStream(DataOutput stream) throws
		IOException;
}	