package xelfi.design.inspector;

import java.awt.*;
import java.util.Vector;

import xelfi.top.*;
import xelfi.design.forms.*;
import xelfi.design.components.*;
import xelfi.design.components.xawt.*;
import xelfi.awt.*;
import xelfi.util.Debug;

/**
 * Main menu editor class. Uses class XFTreeViewPanel to show menu items
 * hierarchically in tree shape. Handles editing, inserting, deleting
 * of menu items.
 *
 * @version 	1.01, 25 Feb 1997
 * @author 		Dafe, Ian Formanek
	Last change:  DF   17 Mar 97   12:24 pm
 */
public class XFMenuEditor extends TopFrame implements XelfiMenuEditor {

	// types of notifications
	private static final int				SELECTION_CHANGE = 1;
	private static final int				PROPERTIES_CHANGE = 2;

	// xelfi design menu bar object
	private XFMenuTreeElement				menu;
	// a frame which will receive created menu
	private Frame										target;
	// tree view panel (tree+scrollbar)
	private XFTreeViewPanel					treeViewPanel;
	// text field for editing items
	private TextField								textField;
	// edit mode/view mode
	private boolean									editMode;
	// reference to Component Inspector
	private XelfiComponentInspector	ci;
  // empty menu - helper (around the win95 bug)
  // win95 bug is that we can't chnage the properties 
  // of menu bar which is set into some Frame
  private MenuBar                 emptyMenu;
  // default size of text field - only heght is important for us
  private Dimension               defaultTextFieldSize = new Dimension(0, 20);


	public XFMenuEditor (Frame target) {
		super();
    setTitle("Menu Editor");
    setBackground(Color.lightGray);
		editMode = false;		

    // initialize empty menu (one item called "empty")
    emptyMenu = new XMenuBar();
		// get reference to CI
		ci = getComponentInspector();
    // set absolute positioning
    setLayout(null);
    // create text field and hide it initially
    textField = new TextField();
    add(textField);
    textField.hide();
		// create tree view
    treeViewPanel = new XFTreeViewPanel(
                        new XFMenuTreeElement(emptyMenu));
    add(treeViewPanel);
		// enable drag-drop operations on the tree
		// treeViewPanel.setDragDrop(true);
		// set its own command menu bar
		MenuBar ownMB = new MenuBar();
		fillOwnMenuBar(ownMB);
		// set menubar
		setTarget(target);
	}
	
	public XFMenuEditor () {
		// call parametrized constructor, setting ourself as target
		this(null);
	}

	/**
	 * Only for testing purposes......
	*/
	public XFMenuEditor (XelfiComponentInspector ci, int foo) {
		this(null);
		this.ci = ci;
	}

	public void setMenu (XMenuBar xmb) {
		boolean canChange = checkTarget();
		if (canChange)
	    removeMenu();
//    if (treeViewPanel!=null) remove(treeViewPanel);
//    if (textField!=null) remove(textField);
    createMenuTree(true, xmb);
//    add(textField);
//    add(treeViewPanel);
		if (canChange)
	    setMenu();
    repaint();
	}

	/**
		* Returns currently asociated MenuBar object.
	*/
	public XMenuBar getMenu () {
		return menu==null ? null : (XMenuBar)menu.getMenuComponent();
	}

	/**
	 * Sets new target - frame which will receive menu
	 * Removes menu from old target.
	*/
	public void setTarget (Frame target) {
//		if (this.target!=null)
//			this.target.setMenuBar(null);
		this.target = target;
    if ((checkTarget())&&(menu!=null))
			this.target.setMenuBar((MenuBar)menu.getMenuComponent());
	}

	/**
	 * Returns current target.
	*/
	public Frame getTarget () {
		return target;
	}

	/**
	 * Returns currently selected xelfi menu component
	 * (vector with only one element)
	*/
	public Vector getSelectedComponents() {
    if (menu==null) return null;
		Vector selComps = new Vector();
		selComps.addElement((Object)(getCurrentMTE().getMenuComponent()));
		return selComps;
	}

	/**
	 * Does a nothing at this time.
	*/
	public void setSelectedComponents(Vector comps) {
		if (comps.size()!=1) return;
		// find a node which holds specified component
		XFTreeNode node = treeViewPanel.getTreeViewCanvas().getTreeView().
											getTree().findNodeWithData(this, comps.elementAt(0));
		if (node==null) {
			Debug.debug(Debug.INSPECTOR, "Something's wrong!!! - no node found");
			return;
		}
		treeViewPanel.getTreeViewCanvas().selectAbsNode(node);
	}

	/**
	 * Returns vector of all menu components in menu that's being created
	*/
  public Vector getAllComponents() {
    if (menu==null) return null;
		// traverse the tree (into its depth) and create new vector of its
		// nodes										
		XFTreeNode rootNode = treeViewPanel.getTreeViewCanvas().getTreeView().
													getNode(0);
		Vector allComps = new Vector();
//		Debug.debug(Debug.INSPECTOR, "getAllComps called....");
		createVectorOfComps(rootNode, allComps);
		return allComps;
	}

	/**
	 * Handles incoming key events.
	*/
  public boolean keyDown (Event e, int key) {
		switch (e.key) {
			case 10: // Enter pressed
				if (editMode==false)
					enterEditing();												
				else
					finishEditing();
				break;
			case 27: // Esc pressed
				if (editMode==true) 
					escapeEditing();
        return true; // don't propagate
      case 127: // Del pressed
        if (editMode==false)
          remove();
        break;
      case 9: // Ctrl+I pressed = insert item
        if (editMode==false)
          insertItem();
        break;
      case 2: // Ctrl+B pressed = insert checkbox item
        if (editMode==false)
          insertCheckboxItem();
        break;
      case 6: // Ctrl+F pressed = insert menu Folder
        if (editMode==false)
          insertMenu();
        break;
      case 19: // Ctrl+S pressed = insert separator
        if (editMode==false)
          insertSeparator();
        break;
		}
		return false;
	}

	/**
	 * Fills up own menu bar with menu commands and set it.
	*/
	private void fillOwnMenuBar (MenuBar mb) {
		Menu creation = new Menu("Creation");
		creation.add("New menu       Ctrl+F");
		creation.add("New item       Ctrl+I");
		creation.add("New separator  Ctrl+S");
		creation.add(new MenuItem("-"));
		creation.add("Remove         Del");
		mb.add(creation);
		Menu editMenu = new Menu("Edit");
		editMenu.add("Edit         Enter");
		editMenu.add("Escape       Esc");
		editMenu.add(new MenuItem("-"));
		editMenu.add("Toggle pull  Space");
		mb.add(editMenu);
		setMenuBar(mb);
	}

	/**
	 * Responds to the menu commands.
	*/
  public boolean action(Event e, Object arg) {
//		Debug.debug(Debug.INSPECTOR, "menu clicked..");
    if (e.target instanceof MenuItem)
    {
      String label = (String) arg;
      if (label.equalsIgnoreCase("New menu       Ctrl+F"))
				insertMenu();
      else 
			if (label.equalsIgnoreCase("New item       Ctrl+I"))
				insertItem();
			else
			if (label.equalsIgnoreCase("New separator  Ctrl+S"))
				insertSeparator();
			else
			if (label.equalsIgnoreCase("Remove         Del"))
				remove();
      else
			if (label.equalsIgnoreCase("Edit         Enter")) {
				if (editMode==false) enterEditing();
			}
      else
			if (label.equalsIgnoreCase("Escape       Esc")) {
				if (editMode==true) escapeEditing();
			}
      else
			if (label.equalsIgnoreCase("Toggle pull  Space")) {
				if (editMode==false) treeViewPanel.togglePull();
			}
		}
		return false;
	}

	public boolean handleEvent (Event e) {
		boolean result = super.handleEvent(e);

		switch (e.id) {
			// notification messages from tree view
			case XFTreeViewCanvas.DRAG_DROP:
				Debug.debug(Debug.INSPECTOR, "drag-drop operation catched");
				XelfiDragDropInfo ddInfo = (XelfiDragDropInfo)e.arg;
				changeParent((XFMenuTreeElement)ddInfo.getSource(),
                     (XFMenuTreeElement)ddInfo.getDestination(),
                     (XFMenuTreeElement)ddInfo.getPreviousItem());
				break;
			case XFTreeViewCanvas.SELECTION_CHANGED:
				Debug.debug(Debug.INSPECTOR, "notifying inspector");
				notifyInspector(SELECTION_CHANGE);
				break;
		}
		return result;
	}

	public void paint (Graphics g) {
		Dimension mySize = size();
		// get the insets
		Insets insets = insets();
		treeViewPanel.reshape(insets.left, insets.top, 
														mySize.width-(insets.left+insets.right), 
														mySize.height-(insets.top+insets.bottom));
		treeViewPanel.layout();
		if (editMode==true) {
			paintTextField(g, insets);
		}
	}

	/**
	 * Inserts new menu item into currently selected item
	*/
	public void insertItem () {
		if (editMode==true) return;
		// can't insert items into main menubar
		if (((XFMenuTreeElement)getCurrentNode().getParent().getData())==menu) 
			return;
    XMenuItem newItem = new XMenuItem("");
    ((XelfiDesignForm)target).setComponentName(newItem, "");
    setSameParent(newItem);
		if (insertAfter(new XFMenuTreeElement(newItem))==false) return;
		// make currently inserted item visible
		treeViewPanel.makeCurrentVisible();
		enterEditing();
	}

	/**
	 * Inserts new menu item into currently selected item
	*/
	public void insertCheckboxItem () {
		if (editMode==true) return;
		// can't insert items into main menubar
		if (((XFMenuTreeElement)getCurrentNode().getParent().getData())==menu) 
			return;
    XCheckboxMenuItem newItem = new XCheckboxMenuItem("");
    ((XelfiDesignForm)target).setComponentName(newItem, "");
    setSameParent(newItem);
		if (insertAfter(new XFMenuTreeElement(newItem))==false) return;
		// make currently inserted item visible
		treeViewPanel.makeCurrentVisible();
		enterEditing();
	}

	/**
	 * Inserts new menu item into currently selected item
	*/
	public void insertMenu () {
		if (editMode==true) return;
    XMenuFolder newFolder = new XMenuFolder("");
    ((XelfiDesignForm)target).setComponentName(newFolder, "");
		// can't insert after root
		if (insertAfter(new XFMenuTreeElement(newFolder))==false) return;
    setSameParent(newFolder);
    XMenuItem newItem = new XMenuItem("item");
    ((XelfiDesignForm)target).setComponentName(newItem, "");
    setSameParent(newItem);
		insertInto(new XFMenuTreeElement(newItem));
		treeViewPanel.previous();
		// make currently inserted item visible
//		treeViewPanel.makeCurrentVisible();
		enterEditing();
	}

	/**
	 * Inserts new menu item into currently selected item
	 * Returns false if operation fails
	*/
	public void insertSeparator () {
		// can't insert separators into main menubar
		if (((XFMenuTreeElement)getCurrentNode().getParent().getData())==menu) 
			return;
    XMenuItem newSep = new XMenuItem("-");
    ((XelfiDesignForm)target).setComponentName(newSep, "");
    setSameParent(newSep);
		insertAfter(new XFMenuTreeElement(newSep));
		// make currently inserted item visible
		treeViewPanel.makeCurrentVisible();
	}

	/**
	 * Removes an item or submenu from menu.
	 * Fails if we're trying to remove last item of submenu 
	*/
	public void remove () {
		if (editMode==true) return;
		XFTreeNode curNode = treeViewPanel.getTreeViewCanvas().
												 getTreeView().getCurNode();
		// can't remove root 
		if (curNode.getParent()==null) return;
		// can't remove last item in submenu
		if (curNode.getParent().getChildCount()<=1) return;
		// yep - now remove from tree and from menu
		XFMenuTreeElement mte = (XFMenuTreeElement)curNode.getData();
    // release component name
		((XelfiDesignForm)target).
				setComponentName((XelfiDesignComponent)mte.getMenuComponent(),
												  null);
		treeViewPanel.remove();
		mte.remove();
		treeViewPanel.makeCurrentVisible();
	}

	/**
	 * Implementation of method from XelfiElementComparer interface.
	 * defines a way how to compare menu component and menu tree elements.
	*/
	public boolean compare (Object first, Object second) {
		return ((XFMenuTreeElement)first).getMenuComponent().equals(second);
	}

  /**
   * Removes menu from target.
   * This method is obviously set before change of properties
   * of menu components can occur.
  */
  public void removeMenu () {
    if ( target==null ) return;
    target.setMenuBar(emptyMenu);
  }

  /**
   * Sets the menu to the target frame. 
   * Usually called after change of menu items' properties.
  */
  public void setMenu () {
    if ( target==null ) return;
    target.setMenuBar((MenuBar)menu.getMenuComponent());
  }

  public boolean gotFocus (Event e, Object what) {
    boolean result = super.gotFocus(e, what);
    // set ourself as inspectable target for CI
    if ( ci.getInspectable()!=this )
      ci.inspectableChanged(this);
    return result;
  }

	/**
	 * Refreshes content of the tree.
	*/
	public void refresh () {
		treeViewPanel.refresh();
	}

	/**
	 * Performs the work of inserting new item after currently selected item.
	 * if it is possible
	*/
	private boolean insertAfter (XFMenuTreeElement item) {
		// insert new item into the tree
		XFMenuTreeElement current = getCurrentMTE();
		// can't insert after the root element
		if (current==menu) return false;
		treeViewPanel.insertAfter(item);
		// and insert into the menu 
    // !!menu must be removed if we want to change its properties!!
    // program crashes on Win95 otherwise....:-((
		boolean canChange = checkTarget();
		if (canChange)
			removeMenu();
		current.insertAfter(item);	
    // pain, pain - set the menu again
		if (canChange)
		  setMenu();
		// notify inspector
		notifyInspector(SELECTION_CHANGE);
		return true;
	}

	/**
	 * Performs the work of inserting new item into currently selected,
	 * if it is possible
	*/
	private void insertInto (XFMenuTreeElement item) {
		// insert new item and menu element 
		XFMenuTreeElement current = getCurrentMTE();
		treeViewPanel.insertInto(item);
    // !!menu must be removed if we want to change its properties!!
    // program crashes on Win95 otherwise....:-((
 		boolean canChange = checkTarget();
		if (canChange)
			removeMenu();
		current.insertInto(item);
		if (canChange)
	    setMenu();
		// notify inspector
		notifyInspector(SELECTION_CHANGE);
	}

	/**
	 * Paints text field at position of currently selected item in tree.
	 * Sets focus to text field.
	*/
	private void paintTextField (Graphics g, Insets insets) {
		// find out position of selected item in tree
		Rectangle bounds = 
						treeViewPanel.getElementBounds(treeViewPanel.getCurIndex());
//		Debug.debug(Debug.INSPECTOR, "bounds are: "+bounds.toString());
		// paint text field
    Dimension textSize = textField.preferredSize();
//    Debug.debug(Debug.INSPECTOR, "I'm in paintTextField...");
    if (bounds==null) {
      Debug.debug(Debug.INSPECTOR, "Something is bad....");
    }
		if (bounds.height<textSize.height)
			textField.reshape(bounds.x+insets.left, 
												bounds.y+insets.top-((textSize.height-bounds.height)/2), 																								bounds.width+50, textSize.height);
		else
			textField.reshape(bounds.x+insets.left, bounds.y+insets.top, 
												bounds.width<20 ? bounds.width+100 : bounds.width+30,
												textSize.height);
		textField.requestFocus();
	}


	/**
	 * Enters edit mode - shows edit control, disables other user input
	*/
	private void enterEditing () {
		editMode = true;
		treeViewPanel.enableUserInput(false);
		// retrieve string from currently selected item 
		// and set it to textField
		XFMenuTreeElement item = getCurrentMTE();
		textField.setText(item.getText());
		textField.show();
		repaint();															
	}

	/**
	 * Finishes edit mode - has to hide edit control and save 
	 * its content into selected tree element. 
	*/
	private void finishEditing () {
		editMode = false;
		treeViewPanel.enableUserInput(true);		
		// retrieve string from text field  
		// and set it to tree item and to the menu
    // do not forget to remove menu at first
		boolean canChange = checkTarget();
		if (canChange)
	    removeMenu();
		XFMenuTreeElement item = getCurrentMTE();
		item.setText(textField.getText());
		if (canChange)
	    setMenu();
    textField.hide();
		treeViewPanel.refresh();
//		repaint();
		// let ci know that properties has changed
		notifyInspector(PROPERTIES_CHANGE);
		// set focus back to the treeview
		treeViewPanel.getTreeViewCanvas().requestFocus();
	}

	/**
	 * Escapes edit mode - hide edit control, don't change any value
	*/
	private void escapeEditing () {
		editMode = false;
		textField.hide();
		treeViewPanel.enableUserInput(true);
		repaint();
		treeViewPanel.getTreeViewCanvas().requestFocus();
	}

	private void resetFromMenu (XMenuBar menu) {
	}

	/**
	 * Refreshes menu in target frame by setting it again
	*/
	private void refreshMenu () {
		if (target!=null)
			target.setMenuBar((MenuBar)menu.getMenuComponent());
	}

	/**
	 * Returns currently selected Menu Tree element.
	 * BEWARE! This code is bad bad bad and ugly :-(
	 * menu editor should not be coupled with so many other objects :-(
	*/
	private XFMenuTreeElement getCurrentMTE () {
		XFTreeNode curNode = getCurrentNode();
		return curNode==null ? null 
												 : (XFMenuTreeElement)(curNode.getData());
	}

	private XFTreeNode getCurrentNode () {
		return treeViewPanel.getCurNode();
	}

	private void selectNode (XFTreeNode node) {
		treeViewPanel.selectNode(node);
	}

	/**
	 * Constructs vector of menu components
	*/
	private void createVectorOfComps (XFTreeNode node, Vector comps) {
		// add node into vector
//		Debug.debug(Debug.INSPECTOR, ((XFMenuTreeElement)node.getData()).getMenuComponent().toString());
		comps.addElement(((XFMenuTreeElement)node.getData()).getMenuComponent());
		// recursive call on each children
		for (int i=0; i<node.getChildCount(); i++)
			createVectorOfComps(node.getChild(i), comps);
	}

	/**
	 * access method to CI.
	*/
	private XelfiComponentInspector getComponentInspector () {
		return TopLevel.getInspector();
	}

	private void notifyInspector (int type) {
		if (ci==null) return;
		switch (type) {
			case SELECTION_CHANGE:
				ci.selectionChanged();
				break;
			case PROPERTIES_CHANGE:
				ci.propertiesChanged();
				break;
		}
	}

  /**
   * Creates tree of menu tree components from 
   * given xmenubar
  */
  private XFMenuTreeElement createMenuTree (boolean isFirst,
                                            MenuComponent mc) {
    XFMenuTreeElement current = new XFMenuTreeElement(mc);
    if ( mc instanceof XMenuBar ) {
      XMenuBar xmb = (XMenuBar)mc;
      menu = current;
      //treeViewPanel = new XFTreeViewPanel(menu);
      treeViewPanel.newRoot(menu);
//			treeViewPanel.setMultipleSelections(true);
//			treeViewPanel.selectNode(0);
      // recurse to the children
 //     XFMenuTreeElement prevElement = null;
      for (int i=0; i<xmb.countMenus(); i++) {
        createMenuTree(i==0, xmb.getMenu(i));
      }
    } else {
      if ( mc instanceof XMenuFolder ) {
        XMenuFolder xmf = (XMenuFolder)mc;
        // add itself
        if ( isFirst ) {
          treeViewPanel.insertInto(current);
//          previous.insertInto(current);
        } else {
          treeViewPanel.insertAfter(current);
//          previous.insertAfter(current);
        }
        // save current node 
        XFTreeNode savedNode = getCurrentNode();
        // recurse to the children
//        XFMenuTreeElement prevEl = null;
        for (int i=0; i<xmf.countItems(); i++) {
          createMenuTree(i==0, xmf.getItem(i));
        }
        // select saved node back (itself)
        // must be here to ensure good work of 
        // treeViewPanel.insertAfter...
//				treeViewPanel.deselectAll();
        selectNode(savedNode);
      } else {
        if (( mc instanceof XMenuItem ) || 
             ( mc instanceof XCheckboxMenuItem )) {
          // add itself
          if ( isFirst ) {
//            previous.insertInto(current);
            treeViewPanel.insertInto(current);
          } else {
//            previous.insertAfter(current);
            treeViewPanel.insertAfter(current);
          }
        }
      }
    }
    return current;
  }

  /**
   * Sets the parent form of the component to be 
   * the same as in main MenuBar component
  */
  private void setSameParent (XelfiDesignComponent comp) {
    // retrieve the parent form
    XelfiDesignForm parentForm =
        ((XMenuBar)menu.getMenuComponent()).getParentForm();
    // and set it 
    comp.setParentForm(parentForm);
  }

	/**
	 * Checks current target to see if it has the same menu
	 * as we. Returns true/false.
	*/
	private boolean checkTarget () {
		if (target==null) return false;
		return ((MenuBar)menu.getMenuComponent())==target.getMenuBar();
	}

	/**
	 * Changes the parent of the menu item contained in source 
	 * to the destination's menu container, if it is possible
	*/
	private void changeParent (XFMenuTreeElement source, 
                             XFMenuTreeElement destination,
                             XFMenuTreeElement after) {
 		boolean canChange = checkTarget();
		if (canChange)
			removeMenu();
		// delete source from its parent
		source.remove();
    // add source to the destination after specified item
    if (after==null) {
      destination.insertAsFirst(source);
    } else {
      after.insertAfter(source);
    }
		if (canChange)
	    setMenu();
	}

}

