/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import xelfi.editor.keys.KeyTable;
import xelfi.editor.keys.KeyOperator;
/**
* Component that allows editing of text documents
*
* @version 1.0
* @author Jaroslav Tulach
*/

final class EditorCanvas extends Canvas implements TextEditorInfo {
  /** the document to display */
  private Document doc;

  /** table of keys */
  private KeyTable keyTable = KeyOperator.newTable ();

  /** selected block */
  Block select;

  /** the position where the block extending started */
  Position mark;

  /** hashtable for properties, like position and so on */
  Hashtable hashTable = new Hashtable ();

  /** draw object */
  EditorImage draw;

  /** cursor */
  Position cursor;

  /** edit info */
  EditorCallback info;

  /** undo of document */
  Undo _undo;

  /** cursor listeners */
  Vector curList = new Vector ();

  /** editor listener */
  Vector edList = new Vector ();


  /** Adds new cursor listener.
  * @param l the listener
  * @see CursorListener
  */
  public void addCursorListener (CursorListener l) {
    curList.addElement (l);
  };

  /** Adds new editor listener.
  * @param l the listener
  * @see EditorListener
  */
  public void addEditorListener (EditorListener l) {
    edList.addElement (l);
  };

  /** Removes listener.
  * @param l the listener
  * @return true if l was in the listener list
  * @see CursorListener
  */
  public boolean removeCursorListener (CursorListener l) {
    return curList.removeElement (l);
  };

  /** Removes listener.
  * @param l the listener
  * @return true if l was in the listener list
  * @see EditorListener
  */
  public boolean removeEditorListener (EditorListener l) {
    return edList.removeElement (l);
  };

  /** Construct new editor with specified document. As a callback default
  * implementation is used.
  *
  * @param d the document
  * @param u should we record events for undo?
  */
  public EditorCanvas (Document d, boolean u) {
    doc = d;
    setBackground (Color.black);
    draw = new EditorImage (this);
    cursor = doc.getBegin ();
    try {
      select = new Block (cursor, cursor);
    } catch (InvalidPositionException ex) {
    };
    info = null;
    if (u) {
      _undo = _undo.getUndo (d);
    };
  };

  /** Sets new editor callback
  * @param c the callback
  */
  public void setEditorCallback (EditorCallback c) {
    info = c;
  };

  /** Only returns editor callback.
  * @return the editor callback
  */
  public EditorCallback getEditorCallback () {
    return info;
  };

  /** paint */
  public void paint (Graphics g) {
    update (g);
  };

  /** update do not clears background */
  public void update (Graphics g) {
    g.drawImage (draw.getImage (), 0, 0, this);
  };

  /** close */
  public void close () {
    draw.close ();
  };

  /** Gets the document in this window.
  * @return the document
  */
  public final Document getDocument () {
    return doc;
  };

  /** Gets the current position in document
  * @return the position
  */
  public final Position getCursorPos () {
    return cursor;
  };

  /** Sets the cursor. If it is out of screen then changes the screen view.
  * @param pos the position to set the cursor to
  */
  public final void setCursorPos (Position pos) {
    if (pos.sameLine (draw.getEnd ())) {
      // only one line after the visible screen
      Position endOfDoc = pos.getDocument ().getEnd ();
      if (
        !pos.sameLine (endOfDoc) ||
        draw.lineToInt (endOfDoc) >= numberOfVisibleLines ()
      ) {
        // the end of document
        try {
          scrollDown ();
        } catch (EndOfTextException ex) {
        };
      };
    } else {
      if (!draw.containsOrBeginsOrEnds (pos.beginOfLine ())) {
        if (pos.safeDown (1).sameLine (draw.getBegin ())) {
          // only one line up, means to scroll down
          try {
            scrollUp ();
          } catch (EndOfTextException ex) {
          };
        } else {
          // move the cursor
          setVisibleBlockBegin (pos.safeUp (draw.visibleLinesCount () / 2));
        };
      };
    };
    setLocalCursor (pos);
  };

  /** Sets the cursor. Do not change screen view.
  * @param pos the position to set the cursor to
  */
  private final void setLocalCursor (Position pos) {
    Position old = cursor;
    draw.drawCursor ();
    draw.makeHorizontalVisible (cursor = pos);
    draw.drawCursor ();
//    draw.drawTwoPositions (old, pos);
    // inform scrollbar that position of cursor has changed
    fireCursorMoved ();
  };

  /** Notify cursor position change.
  */
  private final void fireCursorMoved () {
    Enumeration e = curList.elements ();
    while (e.hasMoreElements ()) {
      ((CursorListener)(e.nextElement ())).cursorMoved (this);
    };
  };
  
  /** Fires not allowed action.
  */
  private final void fireEditorNotAllowedAction () {
    Enumeration e = edList.elements ();
    while (e.hasMoreElements ()) {
      ((EditorListener)(e.nextElement ())).editorNotAllowedAction (this);
    };
  };

  /** Fires double click.
  */
  private final void fireEditorDoubleClick () {
    Enumeration e = edList.elements ();
    while (e.hasMoreElements ()) {
      ((EditorListener)(e.nextElement ())).editorDoubleClick (this, getCursorPos ());
    };
  };
  
  /** Fires view changed.
  */
  private final void fireEditorViewChanged () {
    Enumeration e = edList.elements ();
    while (e.hasMoreElements ()) {
      ((EditorListener)(e.nextElement ())).editorViewChanged (this);
    };
  };

  /** Scrolls up. */
  public void scrollUp () throws EndOfTextException {
    draw.scrollUp ();
    fireEditorViewChanged ();
  };

  /** Scrolls down. */
  public void scrollDown () throws EndOfTextException {
    draw.scrollDown ();
    fireEditorViewChanged ();
  };

  /** Captures mouse event to position a cursor.
  */
  public boolean mouseDown (Event e, int x, int y) {
    requestFocus ();
    if ((e.modifiers & (Event.META_MASK | Event.ALT_MASK)) == 0) try {
      // only works for left mouse button
      Position p = draw.pointToPosition (x, y);
      setLocalCursor (p);
      if (e.clickCount % 3 == 0) {
        Position xxx = p.beginOfLine ();
        setSelectedBlock (Block.between (xxx, xxx.safeDown (1)));
      };
      if (e.clickCount % 2 == 0) {
        // double click
        if (Character.isLetterOrDigit (p.getChar ())) {
          // no one handled it => select the word under cursor
          try {
            Acceptor ac = new AcceptorSpace ();
            Position b = p.find (ac, false);
            Position end = p.find (ac, true);
            setSelectedBlock (Block.between (b.nextChar (), end));
          } catch (EndOfTextException ex) {
          };
        };
        fireEditorDoubleClick ();
      };
    } catch (EndOfTextException ex) {
    };
    return true;
  };

  /** Captures mouse event to position a cursor and select block.
  */
  public boolean mouseDrag (Event e, int x, int y) {
    requestFocus ();

    try {
      // only for left mouse key
      if (y < 0) {
        draw.scrollUp ();
        y = 0;
      };

      int h = size ().height;
      if (y > h && !getCursorPos ().isEndOfText ()) {
        draw.scrollDown ();
        y = h - 3;
      };
    } catch (EndOfTextException ex) {
    };

    blockExtendBegin ();
    try {
      setLocalCursor (draw.pointToPosition (x, y));
    } catch (EndOfTextException ex) {
    };
    blockExtendEnd ();

    return true;
  };

  /** Convert key to new values
  * @param key the key
  * @param mod state of modifiers
  * @return new key code
  */
  private static int convertKey (int key, int mod) {
    if ((mod & KeyOperator.CTRL) != 0) {
      // if CTRL that we must add to each key a value to be between
      // 'A' and 'Z'
      if (key < 'z') {
        switch (key) {
        case '\n':
        case '\b':
        case '\t':
          break;
        default:
          key = key + ('A' - 1);
        };
      };
    };
    return key;
  }

  /** Default action. Called when no one wants this key.
  * @param key the key
  * @param mod the state of modifiers, constants KeyOperator.XXXX
  * @return true, if handled
  * @exception NotAllowedException
  * @exception EndOfTextException
  */
  public boolean defaultAction (int key, int mod)
  throws NotAllowedException, EndOfTextException {
    if (
      ((key >= ' ' && key < 127) || key == '\n') &&
      (mod & (KeyOperator.ALT | KeyOperator.CTRL)) == 0
    ) {
      getCursorPos ().insert ((char)key);
      return true;
    } else {
      return false;
    }
  }

  /** redefine key handling method
  * @param e the event that happened
  * @param key the key pressed
  */
  public boolean keyDown (Event e, int key) {
    boolean ctrl = (e.modifiers & Event.CTRL_MASK) != 0;
    boolean shift = (e.modifiers & Event.SHIFT_MASK) != 0;
    boolean alt = (e.modifiers & (Event.META_MASK | Event.ALT_MASK)) != 0;

    int mod =
      (ctrl ? KeyOperator.CTRL : 0) |
      (shift ? KeyOperator.SHIFT : 0) |
      (alt ? KeyOperator.ALT : 0);

    boolean handled = false;
    try {
      key = convertKey (key, mod);
      // handle the key
      if (keyTable.keyDown (key, mod, this)) {
        // we should execute default action
        handled = defaultAction (key, mod);
      } else {
        handled = true;
      };
    } catch (EndOfTextException ex) {
    } catch (NotAllowedException ex) {
      if (ex.isNotify ()) {
        fireEditorNotAllowedAction ();
      };
    };
    setCursorPos (getCursorPos ());
    return handled;
  };


  /** Handle event.
  * @param ev the event
  */
  public boolean handleEvent (Event ev) {
    if (super.handleEvent (ev)) {
      // handled
      return true;
    } else {
      if (ev.id == Event.GOT_FOCUS || ev.id == Event.WINDOW_EXPOSE) {
        keyTable.init ();
        draw.testVisible ();
        return false;
      }
       return false;
    }

  }

  /** Prefered size.
  * @return the prefered size
  */
  public Dimension preferredSize () {
    Dimension d = info.getCharDimension ();
    return new Dimension (
      d.width * Editor.preferredWidth,
      d.height * Editor.preferredHeight
    );
  };

  /** Minimum size.
  * @return the minimum size
  */
  public Dimension minimumSize () {
    return new Dimension (200, 100);
  };

  /** Equals.
  * @param o the second object
  * @return true if this and o references the same editor
  */
  public boolean equals (Object o) {
    if (o instanceof TextEditor) return ((TextEditor)o).equals (this);
    if (o instanceof EditorCanvas) return this == o;
    return false;
  };

  /** Returns block that contains current selection. This block is valid only
  * in the time of question.
  * @return the selected block
  */
  public Block getSelectedBlock () {
    return select;
  };

  /** Changes selected block. Also redraws.
  * @param sel the block to select or null to clear selected block
  */
  public synchronized void setSelectedBlock (Block sel) {
    if (sel == null) {
      Position beg = doc.getBegin ();
      sel = Block.between (beg, beg);
    };
    Block b = select;
    select = sel;
    draw.redrawBlock (select);
    draw.redrawBlock (b);
  };

  /** Get number of visible lines.
  * @return the number of lines that fits into editor window
  */
  public int numberOfVisibleLines () {
    return draw.visibleLinesCount ();
  };

  /** Gets visible block.
  * @return the visible block
  */
  public Block getVisibleBlock () {
    return draw;
  };

  /** Sets visible block begin.
  * @param p position where the visible part should begin
  */
  public void setVisibleBlockBegin (Position p) {
    if (draw.setBegin (p)) {
      repaint ();
      fireEditorViewChanged ();
    };
  };
  
  /** Begins text marking. Only remembers current cursor position.
  */
  public void blockExtendBegin () {
    mark = getCursorPos ();
  };

  /** Ends marking. Must be called after startMark. Changes selected block.
  * Redraws if needed.
  *
  */
  public synchronized void blockExtendEnd () {
    Block b;
    try {
      if (mark.equals (select.getBegin ())) {
        b = Block.between (getCursorPos (), mark);
        select = Block.between (getCursorPos (), select.getEnd ());
        draw.redrawBlock (b);
        return;
      };

      if (mark.equals (select.getEnd ())) {
        b = Block.between (getCursorPos (), mark);
        select = Block.between (select.getBegin (), getCursorPos ());
        draw.redrawBlock (b);
        return;
      };

      // totally new block has been marked
      setSelectedBlock (Block.between(mark, getCursorPos ()));
    } catch (InvalidPositionException ex) {
    }
  }

  /** Returns editor hashtable. This should be used to store
  * information about the editor like special position,
  * find and replace dialog and so on...
  */
  public Hashtable getHashtable () {
    return hashTable;
  };

  /** Gets frame or dialog where this editor is stored.
  * @return the window where this editor is stored
  */
  public Window getWindow () {
    Component c = this;
    for (;;) {
      try {
        return (Window)c;
      } catch (Exception ex) {
      };
      c = c.getParent ();
    }
  }

}








