/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;
/**
* This is the standard implementation of EditorCallback.
* The main purpose of this class is to handle communication between
* different parts of system that influence the color of text.
*
* <H5>How it works?</H5>
* <UL>
* <LI>If the drawn text is selected in the editor the right background and
* foreground colors are used to draw it. These colors can be specified to
* constructor.
* <LI>Otherwise the list of BackgroundCallback is scanned for a background
* color of whole line. If one of them accepts and returns the color it will
* be used to draw the background of line.
* <LI>At last the specified EditorCallback for syntax highlighting is ask for
* the foreground color. But only if syntax is not allowed by BackgroundCallback
* asked in previous step the nonsyntax text is used.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class StandardEditorCallback extends Object implements EditorCallback {
  /** syntax callback */
  EditorCallback syntax;

  /** the canvas to draw on */
  TextEditor editor;

  /** vector of background callbacks */
  Vector background = new Vector ();

  /** the background of selected text */
  public static Color selectBack = Color.gray;

  /** the foreground of selected text */
  public static Color selectFore = Color.black;

  /** the color of normal background */
  public static Color normalBack = Color.black;

  /** the color of normal text */
  public static Color normalFore = Color.white;

  /** the font for selected and without syntax parts of text */
  public static Font font = new Font ("Courier", Font.PLAIN, 14);

  /** Constructor.
  * @param e the editor to color text in
  * @param syntax the syntax callback
  */
  public StandardEditorCallback (
    TextEditor e,
    EditorCallback syntax
  ) {
    this.editor = e;
    this.syntax = syntax;
  }

  /** Adds new @see BackgroundCallback to list of them.
  */
  public void addBackgroundCallback (BackgroundCallback bc) {
    background.addElement (bc);
  }

  /** Changes the syntax callback.
  * @param s the syntax callback
  */
  public void changeSyntaxCallback (EditorCallback s) {
    syntax = s;
  }

  /** Returns the height and width of every character used.
  * @return the height
  */
  public Dimension getCharDimension () {
    return syntax.getCharDimension ();
  }
  
  /** Returns the maximum ascent of any character used in document
  * @return the max ascent in points
  */
  public int getMaxAscent () {
    return syntax.getMaxAscent ();
  }

  /** This method is called when a text has to be redrawn from
  * position p.
  * @param p the position of text that should be redrawn
  * @return normal TextDrawInfo extended with the information
  * in the field first that indicates the first position where
  * the text drawing should start (usually the same as p).
  */
  public StartDrawInfo beginDraw (Position p) {
    StandardDrawInfo sdi = new StandardDrawInfo (p);
    
    if (getBackground (sdi)) {
      // we should use syntax coloring
      StartDrawInfo synt = syntax.beginDraw (p);
      sdi.syntax = synt;
      // copy values from the syntax browser
      sdi.end = sdi.first = synt.first;
      setValues (sdi);
    } else {
      // no syntax coloring
      sdi.syntax = null;
      setValues (sdi);
    }
    return sdi;
  }
  
  
  /** This method is called when a new text has been
  * inserted between n and p.
  *
  * z@param n the begin of inserted text
  * @param p the end of inserted text
  * @return information about where to start redrawing (field
  * first - usually before n), where to stop redrawing (field
  * last - should be after p) and the information about how
  * to draw the first part of text
  * @warning the first and last positions should be on the same line
  * as positions n and/or p.
  * @see TextDrawInfo
  */
  public StartDrawInfo beginInsert (Position n, Position p) {
    StandardDrawInfo sdi = new StandardDrawInfo (n);
    
    if (getBackground (sdi)) {
      // we should use syntax coloring
      // copy values from the syntax browser
      if (!n.sameLine (p)) {
        // redraw all
        StartDrawInfo synt = syntax.beginDraw (n.beginOfLine ());
        sdi.syntax = synt;
        sdi.first = synt.first;
        sdi.last = p.endOfLine ();
      } else {
        // do not redraw all
        StartDrawInfo synt = syntax.beginInsert (n, p);
        sdi.syntax = synt;
        sdi.first = synt.first;
        sdi.last = synt.last;
      }
      sdi.end = sdi.first;
      setValues (sdi);
    } else {
      // no syntax coloring
      if (!n.sameLine (p)) {
        // we need to redraw rest of line
        // because lines has been concatenated
        sdi.first = n.beginOfLine ();
        sdi.last = p.endOfLine ();
      } else {
        // delete only on one line, no need to redraw
        sdi.first = n;
        sdi.last = p;
      }
      sdi.syntax = null;
      setValues (sdi);
    }
    return sdi;
  }
  
  /** This method is called when a text s is deleted
  * from position p.
  *
  * @param p the position
  * @param s the deleted text
  * @return the information about where to start, where to
  * finish redrawing and which style to use at the
  * first part of text.
  * @warning the first and last positions should be on the same line
  * as position n.
  */
  public StartDrawInfo beginDelete (Position p, String s) {
    StandardDrawInfo sdi = new StandardDrawInfo (p);

    if (getBackground (sdi)) {
      // we should use syntax coloring
      if (s.indexOf ('\n') >= 0) {
        // redraw all
        p = p.beginOfLine ();
        sdi.syntax = syntax.beginDraw (p);
        sdi.first = p;
        sdi.last = p.endOfLine ();
      } else {
        // do not redraw all
        StartDrawInfo synt = syntax.beginDelete (p, s);
        sdi.syntax = synt;
        sdi.first = synt.first;
        sdi.last = synt.last;
      }
      sdi.end = sdi.first;
      setValues (sdi);
    } else {
      // no syntax coloring
      if (s.indexOf ('\n') >= 0) {
        // we need to redraw rest of line
        // because lines has been concatenated
        sdi.first = p.beginOfLine ();
        sdi.last = p.endOfLine ();
      } else {
        // delete only on one line, no need to redraw
        sdi.last = sdi.first;
      }
      sdi.syntax = null;
      sdi.end = sdi.first;
      setValues (sdi);
    }
    return sdi;
  }

  /** Sets values. In respect to selected block.
  * @param sdi the standard draw info
  */
  private void setValues (StandardDrawInfo sdi) {
    // begin position
    sdi.prev = sdi.end;
    // selected block
    Block sel = editor.getSelectedBlock ();

    if (sel.containsOrBegins (sdi.prev)) {
      // we are in the selected block
      // so redraw till the end
      sdi.end = sel.getEnd ();
      sdi.text = selectFore;
      sdi.back = selectBack;
      sdi.font = font;
      return;
    }
    
    if (sdi.syntax == null) {
      // no syntax coloring
      sdi.end = sdi.prev.beginOfLine ().safeDown (1);
      sdi.text = sdi.defaultText;
      sdi.back = sdi.defaultBack;
      sdi.font = font;
    } else {
      // use syntax coloring
      if (sdi.prev.sameLine (sdi.syntax.end)) {
        sdi.end = sdi.syntax.end;
      } else {
        sdi.end = sdi.prev.beginOfLine ().safeDown (1);
      }
      sdi.text = sdi.syntax.text;
      sdi.back = sdi.defaultBack;
      sdi.font = sdi.syntax.font;
    }

    // test if the end is not after the begin of line
    Position sb = sel.getBegin ();
    if (sdi.prev.sameLine (sb) && sdi.prev.less (sb) && sdi.end.greater (sb)) {
      sdi.end = sb;
    }
  }
  
  /** Returns info about how the text from position p should be painted.
  * @param di the previous draw info
  * @return the info about the text
  */
  public TextDrawInfo getDrawInfo (TextDrawInfo di) {
    try {
      StandardDrawInfo sdi = (StandardDrawInfo)di;

      if (!sdi.end.sameLine (sdi.prev)) {
        // new line
        if (getBackground (sdi)) {
          // we should use syntax coloring
          if (sdi.syntax != null && sdi.end.equals (sdi.syntax.end)) {
            // synchronzied, only continue
            sdi.syntax = syntax.getDrawInfo (sdi.syntax);
            setValues (sdi);
          } else {
            // positions not synchronized, start new draw
            sdi.syntax = syntax.beginDraw (sdi.end);
            setValues (sdi);
          }
        } else {
          // no syntax coloring
          sdi.syntax = null;
          setValues (sdi);
        }
      } else {
        // it has already been deternined if this line has
        // syntax coloring or not
        if (sdi.syntax == null) {
          // there should be no syntax on this line
          setValues (sdi);
        } else {
          // use syntax coloring
          if (sdi.end.equals (sdi.syntax.end)) {
            // requested position is the same as the position in syntax colorer
            // so only ask for next info
            sdi.syntax = syntax.getDrawInfo (sdi.syntax);
            setValues (sdi);
          } else {
            // positions are not synchronized restart new read
            sdi.syntax = syntax.beginDraw (sdi.end);
            setValues (sdi);
          }
        }
      }
    } catch (ClassCastException ex) {
      di.end = di.end.beginOfLine ().safeDown (1);
    }
    return di;
  }

  /** Returns default background that will be used in the editor's canvas.
  * Returning this value can avoid flickering in editor window.
  * @return the color or null
  */
  public Color getDefaultBackground () {
    return normalBack;
  }

  /** Finds valid BackgroundCallback and sets its value to the
  * text info.
  * @param di the draw info
  * @return true if we should use the syntax coloring
  */
  private final synchronized boolean getBackground (StandardDrawInfo di) {
    try {
      Enumeration en = background.elements ();
      for (;;) {
        BackgroundCallback bg = (BackgroundCallback)(en.nextElement ());
        if ((di.defaultBack = bg.getBackground (di.end)) != null) {
          return (di.defaultText = bg.getForeground ()) == null;
        }
      }
    } catch (NoSuchElementException ex) {
      di.defaultBack = normalBack;
      di.defaultText = normalFore;
      return true;
    }
  }
}



