/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
/**
* Standard implemntation of source finder. It scans the CLASSPATH for source file.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class StandardSourceFinder implements SourceFinder {
  /** Finds a source file for the specified class.
  * @param className the name of class to search for
  * @param create If true and the class still do not exist the new source file should be
  * created.
  * @return the source file
  * @exception IOException is thrown either if the class does not exist and create is false
  * or if the class is compiled but sources are unreachable.
  */
  public final XelfiFile findSource (String className, boolean create) throws IOException {
    StandardXelfiFile xf = new StandardXelfiFile ();
    int l = className.lastIndexOf ('.');
    xf.name = className.substring (l + 1);
    xf.pack = l > 0 ? className.substring (0, l) : "";

    String sep = File.pathSeparator;
    char file = File.separatorChar;
    String path = System.getProperty ("java.class.path");
    String kus = xf.pack.replace ('.', file);
    
    // replaces '.' in name by '/' or '\'
    String name = className.replace ('.', file) + ".java";
    String last = "";
    
    StringTokenizer tok = new StringTokenizer (path, sep);
    try {
      for (;;) {
        String next = null;
        next = (last = tok.nextToken ()) + file + name;
        if (!last.trim ().equals ("")) {
          // creates file from
          File f = new File (next);
          // we need a file to read from
          if (f.canRead ()) {
            xf.path = new File (f.getParent ());
            return xf;
          }
        }
      }
    } catch (NoSuchElementException ex) {
      // file has not been found
    }

    if (create) {
      // tries to create the class file
      try {
        Class.forName (className);
        // if class exists and sources do not then
        // throw an exception
      } catch (ClassNotFoundException ex) {
        xf.path = new File (getDefaultXelfiPath () + kus);
        return xf;
      }
    }
    // throw an exception because the file has not been found
    // (and) cannot be created
    throw new FileNotFoundException ("Source has not been found for class " + className);
  }

  /** Gets default path for creating new classes.
  * @return the path with ending (back)slash
  */
  protected String getDefaultXelfiPath () {
    String path = System.getProperty ("java.class.path");
    String sep = File.pathSeparator;
    
    StringTokenizer tok = new StringTokenizer (path, sep);
    try {
      String s= tok.nextToken ();
      return s.charAt (s.length () - 1) == File.separatorChar ? s : s + File.separatorChar;
    } catch (Exception ex) {
      return "";
    }
  }

  /** Tries to find class name for specified file. This is the reverse process to the
  * findSource funciton.
  * @param f the file
  * @return xelfi file
  * @exception IOException if the file is not valid source file
  */
  public XelfiFile findClass (File f) throws IOException {
    StandardXelfiFile xf = new StandardXelfiFile ();
    String fs = f.getAbsolutePath ();
    int lst = fs.lastIndexOf ('.');
    if (f.exists () && lst > 0 && fs.substring (lst).equals (".java")) {
      fs = fs.substring (0, lst);
      
      String sep = File.pathSeparator;
      char file = File.separatorChar;
      String path = System.getProperty ("java.class.path");
  
      String last = null;
      StringTokenizer tok = new StringTokenizer (path, sep);
      try {
        for (;;) {
          File p = new File (tok.nextToken ());
          String ps = p.getAbsolutePath ();
          if (fs.indexOf (ps) == 0) {
            // the file is in this part of class path
            int psl = ps.length ();
            String rest = fs.substring (ps.charAt (psl - 1) == file ? psl + 1 : psl);
            int restl = rest.lastIndexOf (file);
            String kus = restl > 0 ? rest.substring (1, restl) : "";
            xf.pack = kus.replace (file, '.');
            xf.name = rest.substring (restl + 1);
            xf.path = new File (p + kus);
            return xf;
          }
        }
      } catch (NoSuchElementException ex) {
      }
    }
    // file has not been found
    throw new IOException ("File " + f + " is not valid source file");
  }
}


