/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
/**
* This object describes one source in the Xelfi system.
* A lot of useful information can be retrived from the object.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public abstract class XelfiFile extends Object {
  /** Returns the whole path to the file.
  * @return the file of this source
  */
  public abstract File getFile ();

  /** Returns the classpath. The part of path to the file that is
  * contained in the CLASSPATH.
  * @return the class path where the file is stored
  */
  public abstract File getPath ();

  /** Returns the package of the file.
  * @return the package name
  */
  public abstract String getPackage ();

  /** Returns the name of the class.
  * @return only the name of class without the package prefix
  */
  public abstract String getName ();

  /** Returns the whole name of the class.
  * @return the package and class name
  */
  public final String getPackageAndName () {
    String p = getPackage ();
    return p.length () == 0 ? getName () : p + '.' + getName ();
  };

  /** Test if the file exists or not.
  * @return true if the file is on the disk
  */
  public final boolean exists () {
    return getFile ().exists ();
  };

  /** Creates new XelfiFile.
  * @param name the whole name of class (including package)
  * @param create should the source be created if it does not exist
  * @return the xelfi file
  * @exception IOException if an error occures
  */
  public static XelfiFile find (String name, boolean create) throws IOException {
    return Editor.findSource (name, create);
  };

  /** Creates new XelfiFile.
  * @param name the whole name of class (including package)
  * @return the xelfi file
  * @exception IOException if the file does not exists and cannot be created
  */
  public static XelfiFile find (String name) throws IOException {
    return find (name, true);
  };

  /** Creates new XelfiFile.
  * @param pack the name of package
  * @param name the name of class
  * @param create should the source be created if it does not exist
  * @return the xelfi file
  * @exception IOException if an error occures
  */
  public static XelfiFile find (String pack, String name, boolean create) throws IOException {
    return find (pack + '.' + name, create);
  };

  /** Creates new XelfiFile.
  * @param pack the name of package
  * @param name the name of class
  * @return the xelfi file
  * @exception IOException if the class does not exist and cannot be created
  */
  public static XelfiFile find (String pack, String name) throws IOException {
    return find (pack, name, true);
  };

  /** Creates new XelfiFile.
  * @param path the whole path to the file
  * @return the XelfiFile
  * @exception IOException if the path does not contain valid source file
  */
  public static XelfiFile find (File path) throws IOException {
    return Editor.findClass (path);
  };

  /** Converts this object to a string.
  * @return the string
  */
  public String toString () {
    return getPackageAndName () + " [" + getFile () + ']';
  };
};


